import axios from 'axios'
import {
  decryptArrayBuffer,
  encryptArrayBuffer,
  getHash,
  isOnline,
  uploadToFileStorage
} from '../../../utils'
import { MarkStrategy } from '../MarkStrategy'
import { MarkInputSignature } from './Input'
import { MarkRenderSignature } from './Render'

export const SignatureStrategy: MarkStrategy = {
  input: MarkInputSignature,
  render: MarkRenderSignature,
  encryptAndUpload: async (value, encryptionKey) => {
    // Value is the stringified signature object
    // Encode it as text to the arrayBuffer
    const encoder = new TextEncoder()
    const uint8Array = encoder.encode(value)
    const hash = await getHash(uint8Array)

    if (!hash) {
      throw new Error("Can't get file hash.")
    }

    if (!encryptionKey) {
      throw new Error('Signature requires an encryption key')
    }

    // Encrypt the file contents with the same encryption key from the create signature
    const encryptedArrayBuffer = await encryptArrayBuffer(
      uint8Array,
      encryptionKey
    )

    // Create the encrypted json file from array buffer and hash
    const file = new File([encryptedArrayBuffer], `${hash}.json`)

    if (await isOnline()) {
      try {
        const url = await uploadToFileStorage(file)
        console.info(`${file.name} uploaded to file storage`)
        return url
      } catch (error) {
        if (error instanceof Error) {
          console.error(
            `Error occurred in uploading file ${file.name}`,
            error.message
          )
        }
      }
    } else {
      // Handle offline?
    }

    return value
  },
  fetchAndDecrypt: async (value, encryptionKey) => {
    if (!encryptionKey) {
      throw new Error('Signature requires an encryption key')
    }

    const encryptedArrayBuffer = await axios.get(value, {
      responseType: 'arraybuffer'
    })

    const arrayBuffer = await decryptArrayBuffer(
      encryptedArrayBuffer.data,
      encryptionKey
    ).catch((err) => {
      console.log('err in decryption:>> ', err)
      return null
    })

    if (arrayBuffer) {
      // decode json
      const decoder = new TextDecoder()
      const value = decoder.decode(arrayBuffer)
      return value
    }

    // Handle offline?
    return value
  }
}
